// SPDX-License-Identifier: MPL-2.0

import path from "node:path";
import process from "node:process";
import fs, { promises as fsp } from "node:fs";
import filenamify from "filenamify/browser";

export default class Cache {
  constructor(options) {
    this.options = {
      fs: fsp,
      stringify: JSON.stringify,
      parse: JSON.parse,
      cwd: process.cwd(),
      filenamify: filenamify,
      ...options,
    };

    this.cacheDir = path.join(
      this.options.cwd || ".",
      ".cache",
      "asia",
      "spec-tests"
    );

    this.filename = (fp) => {
      return path.join(this.cacheDir, `${this.options.filenamify(fp)}.txt`);
    };

    this.prepare = () => {
      fs.mkdirSync(this.cacheDir, { recursive: true });
    };

    this.filename.bind(this);
    this.prepare.bind(this);

    if (process.env.ASIA_NO_AUTORUN === undefined) {
      this.prepare();
    }
  }

  async set(key, value) {
    await this.options.fs.writeFile(
      this.filename(key),
      this.options.stringify(value)
    );
  }

  async has(key) {
    try {
      await this.options.fs.stat(this.filename(key));
    } catch (err) {
      return false;
    }
    return true;
  }

  async get(key, raw) {
    const value = await this.options.fs.readFile(this.filename(key), "utf-8");

    return raw ? value : this.options.parse(value);
  }

  async clear() {
    try {
      await this.options.fs.rm(path.join(this.options.cwd, ".cache"), {
        recursive: true,
      });
      this.prepare();
    } catch (err) {}
  }
}
